<?php

if (!defined('ABSPATH')) {
    exit('You\'re not allowed to see this page');
} // Exit if accessed directly

class YasrCustomFields extends WP_REST_Controller {

    /**
     * Constructor
     */
    public function __construct() {
        add_action('rest_api_init',  array( $this, 'customFields'));
    }


    /**
     * Add a new response in
     * YOURSITE.COM/wp-json/wp/v2/posts
     */
    public function customFields () {

        /**
         * Add <yasr-visitor-votes> in
         * YOURSITE.COM/wp-json/wp/v2/posts
         */
        $this->visitorVotes();
        $this->allItemTypes();
    }

    /**
     * Helper function to return visitorVotes
     */
    private function visitorVotes() {
        $post_types = yasr_return_all_post_types();
        $yasr_vv_schema = array(
            'description'          => 'Yasr Visitor Votes Data',
            'type'                 => 'object',
            'context'              =>  array('view', 'edit'),
            // In JSON Schema you can specify object properties in the properties attribute.
            'properties'           => array(
                'sum_votes' => array(
                    'type'  => 'integer',
                ),
                'number_of_votes' => array(
                    'type'        => 'integer',
                ),
            ),
        );

        //Register Visitor Votes
        register_rest_field(
            $post_types,
            'yasr_visitor_votes',
            array(
                'get_callback'    => function() {return YasrDatabaseRatings::getVisitorVotes(); },
                'update_callback' => null,
                'schema'          => $yasr_vv_schema
            )
        );

    }

    /**
     * Helper function to return supported itemtype
     */
    private function allItemTypes() {
        $post_types = yasr_return_all_post_types();
        $yasr_itemtype_schema = array(
            'description'          => 'Yasr Supported Item Types',
            'type'                 => 'object',
            'context'              =>  array('edit'),
        );

        //Register Visitor Votes
        register_rest_field(
            $post_types,
            'yasr_all_itemtypes',
            array(
                'get_callback'    => function() {return json_decode(YASR_SUPPORTED_SCHEMA_TYPES); },
                'update_callback' => null,
                'schema'          => $yasr_itemtype_schema
            )
        );
    }
}